"""
Functions about ice and melting, but not the freezing point.
"""

from ._wrapped_ufuncs import (
adiabatic_lapse_rate_ice,
alpha_wrt_t_ice,
chem_potential_water_ice,
cp_ice,
enthalpy_ice,
entropy_ice,
Helmholtz_energy_ice,
ice_fraction_to_freeze_seawater,
internal_energy_ice,
kappa_const_t_ice,
kappa_ice,
melting_ice_equilibrium_SA_CT_ratio,
melting_ice_equilibrium_SA_CT_ratio_poly,
melting_ice_into_seawater,
melting_ice_SA_CT_ratio,
melting_ice_SA_CT_ratio_poly,
melting_seaice_equilibrium_SA_CT_ratio,
melting_seaice_equilibrium_SA_CT_ratio_poly,
melting_seaice_into_seawater,
melting_seaice_SA_CT_ratio,
melting_seaice_SA_CT_ratio_poly,
pot_enthalpy_from_pt_ice,
pot_enthalpy_from_pt_ice_poly,
pressure_coefficient_ice,
pt0_from_t_ice,
pt_from_pot_enthalpy_ice,
pt_from_pot_enthalpy_ice_poly,
pt_from_t_ice,
rho_ice,
seaice_fraction_to_freeze_seawater,
sound_speed_ice,
specvol_ice,
t_from_pt0_ice,
)
